@@ -52,7 +52,7 @@
 #include "soup-input-stream.h"
 
 
-G_DEFINE_TYPE (GVfsBackendHttp, g_vfs_backend_http, G_VFS_TYPE_BACKEND);
+G_DEFINE_TYPE (GVfsBackendHttp, g_vfs_backend_http, G_VFS_TYPE_BACKEND)
 
 static void
 g_vfs_backend_http_finalize (GObject *object)
@@ -67,6 +67,10 @@ g_vfs_backend_http_finalize (GObject *object)
   soup_session_abort (backend->session);
   g_object_unref (backend->session);
 
+  soup_session_abort (backend->session_async);
+  g_object_unref (backend->session_async);
+
+
   if (G_OBJECT_CLASS (g_vfs_backend_http_parent_class)->finalize)
     (*G_OBJECT_CLASS (g_vfs_backend_http_parent_class)->finalize) (object);
 }
@@ -76,11 +80,15 @@ g_vfs_backend_http_init (GVfsBackendHttp *backend)
 {
   g_vfs_backend_set_user_visible (G_VFS_BACKEND (backend), FALSE);  
 
-  backend->session = soup_session_async_new ();
+  backend->session = soup_session_sync_new ();
+  backend->session_async = soup_session_async_new ();
 }
 
+
 SoupURI *
-g_vfs_backend_uri_for_filename (GVfsBackend *backend, const char *filename)
+g_vfs_backend_uri_for_filename (GVfsBackend *backend,
+                                const char  *filename,
+                                gboolean     is_dir)
 {
   GVfsBackendHttp *op_backend;
   SoupURI         *uri;
@@ -95,9 +103,18 @@ g_vfs_backend_uri_for_filename (GVfsBackend *backend, const char *filename)
 
   /* Otherwise, we append filename to mount_base (which is assumed to
    * be a directory in this case).
+   *
+   * Add a "/" in cases where it is likely that the url is going
+   * to be a directory to avoid redirections
    */
-  path = g_build_path ("/", uri->path, filename, NULL);
-  soup_uri_set_path (uri, path);
+  if (is_dir == FALSE || g_str_has_suffix (filename, "/"))
+    path = g_build_path ("/", uri->path, filename, NULL);
+  else
+    path = g_build_path ("/", uri->path, filename, "/", NULL);
+
+  g_free (uri->path);
+  uri->path = g_uri_escape_string (path, G_URI_RESERVED_CHARS_ALLOWED_IN_PATH,
+                                   FALSE);
   g_free (path);
 
   return uri;
@@ -148,6 +165,34 @@ uri_get_basename (const char *uri_str)
 
 /* ************************************************************************* */
 /*  */
+guint
+http_error_code_from_status (guint status)
+{
+  switch (status) {
+
+  case SOUP_STATUS_CANT_RESOLVE:
+  case SOUP_STATUS_CANT_RESOLVE_PROXY:
+    return G_IO_ERROR_HOST_NOT_FOUND;
+
+  case SOUP_STATUS_CANCELLED:
+    return G_IO_ERROR_CANCELLED;
+
+  case SOUP_STATUS_UNAUTHORIZED:
+  case SOUP_STATUS_PAYMENT_REQUIRED:
+  case SOUP_STATUS_FORBIDDEN:
+    return G_IO_ERROR_PERMISSION_DENIED;
+
+  case SOUP_STATUS_NOT_FOUND:
+  case SOUP_STATUS_GONE:
+    return G_IO_ERROR_NOT_FOUND;    
+
+  case SOUP_STATUS_GATEWAY_TIMEOUT:
+    return G_IO_ERROR_TIMED_OUT;
+  }
+
+  return G_IO_ERROR_FAILED;
+}
+
 
 static void
 g_vfs_job_failed_from_http_status (GVfsJob *job, guint status_code, const char *message)
@@ -189,23 +234,31 @@ message_new_from_uri (const char *method,
 }
 
 SoupMessage *
-message_new_from_filename (GVfsBackend *backend,
-                           const char  *method,
-                           const char  *filename)
+message_new_from_filename_full (GVfsBackend *backend,
+                                const char  *method,
+                                const char  *filename,
+                                gboolean     is_dir)
 {
-  GVfsBackendHttp *op_backend;
   SoupMessage     *msg;
   SoupURI         *uri;
 
-  op_backend = G_VFS_BACKEND_HTTP (backend);
-
-  uri = g_vfs_backend_uri_for_filename (backend, filename);
+  uri = g_vfs_backend_uri_for_filename (backend, filename, is_dir);
   msg = message_new_from_uri (method, uri);
 
   soup_uri_free (uri);
   return msg;
 }
 
+
+SoupMessage *
+message_new_from_filename (GVfsBackend *backend,
+                           const char  *method,
+                           const char  *filename)
+{ 
+  return message_new_from_filename_full (backend, method, filename, FALSE);
+}
+
+
 /* ************************************************************************* */
 /* virtual functions overrides */
 
@@ -308,7 +361,7 @@ try_open_for_read (GVfsBackend        *backend,
   op_backend = G_VFS_BACKEND_HTTP (backend);
   msg = message_new_from_filename (backend, "GET", filename);
 
-  stream = soup_input_stream_new (op_backend->session, msg);
+  stream = soup_input_stream_new (op_backend->session_async, msg);
   g_object_unref (msg);
 
   soup_input_stream_send_async (stream,
@@ -359,10 +412,8 @@ try_read (GVfsBackend        *backend,
           char               *buffer,
           gsize               bytes_requested)
 {
-  GVfsBackendHttp *op_backend;
   GInputStream    *stream;
 
-  op_backend = G_VFS_BACKEND_HTTP (backend);
   stream = G_INPUT_STREAM (handle);
 
   g_input_stream_read_async (stream,
@@ -382,11 +433,9 @@ try_seek_on_read (GVfsBackend *backend,
                   goffset    offset,
                   GSeekType  type)
 {
-  GVfsBackendHttp *op_backend;
   GInputStream    *stream;
   GError          *error = NULL;
 
-  op_backend = G_VFS_BACKEND_HTTP (backend);
   stream = G_INPUT_STREAM (handle);
 
   if (!g_seekable_seek (G_SEEKABLE (stream), offset, type,
@@ -444,10 +493,8 @@ try_close_read (GVfsBackend       *backend,
                 GVfsJobCloseRead  *job,
                 GVfsBackendHandle  handle)
 {
-  GVfsBackendHttp *op_backend;
   GInputStream    *stream;
 
-  op_backend = G_VFS_BACKEND_HTTP (backend);
   stream = G_INPUT_STREAM (handle);
 
   g_input_stream_close_async (stream,
@@ -455,7 +502,6 @@ try_close_read (GVfsBackend       *backend,
                               G_VFS_JOB (job)->cancellable,
                               close_read_ready,
                               job);
-
   return TRUE;
 }
 
