@@ -53,11 +53,24 @@ struct _GHalMount {
   GIcon *override_icon;
   GFile *override_root;
   gboolean cannot_unmount;
+  gboolean searched_for_icon;
 
   HalDevice *device;
   HalDevice *drive_device;
 };
 
+static GFile *
+_g_find_file_insensitive_finish (GFile        *parent,
+                                 GAsyncResult *result,
+                                 GError      **error);
+
+static void
+_g_find_file_insensitive_async (GFile              *parent,
+                                const gchar        *name,
+                                GCancellable       *cancellable,
+                                GAsyncReadyCallback callback,
+                                gpointer            user_data);
+
 static void g_hal_mount_mount_iface_init (GMountIface *iface);
 
 #define _G_IMPLEMENT_INTERFACE_DYNAMIC(TYPE_IFACE, iface_init)       { \
@@ -125,7 +138,7 @@ g_hal_mount_init (GHalMount *hal_mount)
 {
 }
 
-const static struct {
+static const struct {
   const char *disc_type;
   const char *icon_name;
   char *ui_name;
@@ -148,7 +161,7 @@ const static struct {
   {"hddvd_r",       "media-optical-hddvd-r", N_("HD DVD-R Disc"), N_("Blank HD DVD-R Disc")},
   {"hddvd_rw",      "media-optical-hddvd-rw", N_("HD DVD-RW Disc"), N_("Blank HD DVD-RW Disc")},
   {"mo",            "media-optical-mo", N_("MO Disc"), N_("Blank MO Disc")},
-  {NULL,            "media-optical", N_("Disc"), N_("Blank Disc")},
+  {NULL,            "media-optical", N_("Disc"), N_("Blank Disc")}
 };
 
 static const char *
@@ -182,6 +195,143 @@ get_disc_name (const char *disc_type, gboolean is_blank)
     return disc_data[n].ui_name;
 }
 
+typedef struct _MountIconSearchData
+{
+  GHalMount *mount;
+  GFile *root;
+} MountIconSearchData;
+
+static void
+clear_icon_search_data (MountIconSearchData *data)
+{
+  if (data->mount)
+    g_object_unref (data->mount);
+  if (data->root)
+    g_object_unref (data->root);
+  g_free (data);
+}
+
+static void
+on_icon_file_located (GObject *source_object, GAsyncResult *res,
+                      gpointer user_data)
+{
+  GFile *icon_file;
+  GIcon *icon;
+  MountIconSearchData *data = (MountIconSearchData *) (user_data);
+  
+  icon_file = _g_find_file_insensitive_finish (G_FILE (source_object),
+                                               res, NULL);
+  
+  /* TODO: check if the file actually exists? */
+
+  icon = g_file_icon_new (icon_file);
+  g_object_unref (icon_file);
+
+  g_hal_mount_override_icon (data->mount, icon);
+  g_object_unref (icon);
+  
+  clear_icon_search_data (data);
+}
+
+static void
+on_autorun_loaded (GObject *source_object, GAsyncResult *res,
+                   gpointer user_data)
+{
+  gchar *content, *relative_icon_path = NULL;
+  gsize content_length;
+  MountIconSearchData *data = (MountIconSearchData *) (user_data);
+  
+  if (g_file_load_contents_finish (G_FILE (source_object), res, &content,
+                                   &content_length, NULL, NULL))
+    {
+      /* Scan through for an "icon=" line. Can't use GKeyFile,
+       * because .inf files aren't always valid key files
+       **/
+      GRegex *icon_regex;
+      GMatchInfo *match_info;
+      
+      /* [^,] is because sometimes the icon= line
+       * has a comma at the end
+       **/
+      icon_regex = g_regex_new ("icon=([^,\\r\\n]+)",
+                                G_REGEX_CASELESS, 0, NULL);
+      g_regex_match (icon_regex, content, 0,
+                     &match_info);
+      
+      /* Even if there are multiple matches, pick only the
+       * first.
+       **/
+      if (g_match_info_matches (match_info))
+        {
+          gchar *chr;
+          gchar *word = g_match_info_fetch (match_info, 1);
+          
+          /* Replace '\' with '/' */
+          while ((chr = strchr (word, '\\')) != NULL)
+            *chr = '/';
+          
+          /* If the file name's not valid UTF-8,
+           * don't even try to load it
+           **/
+          if (g_utf8_validate (word, -1, NULL))
+            relative_icon_path = word;
+          else
+            g_free (word);
+        }
+      
+      g_match_info_free (match_info);
+      
+      g_regex_unref (icon_regex);
+      g_free (content);
+    }
+
+  /* some autorun.in points to the .exe file for the icon; make sure we avoid using that */
+  if (relative_icon_path && !g_str_has_suffix (relative_icon_path, ".exe"))
+    {
+      _g_find_file_insensitive_async (data->root,
+                                      relative_icon_path,
+                                      NULL, on_icon_file_located,
+                                      data);
+      
+      g_free (relative_icon_path);
+    }
+  else
+    clear_icon_search_data (data);
+}
+
+static void
+on_autorun_located (GObject *source_object, GAsyncResult *res,
+                    gpointer user_data)
+{
+  GFile *autorun_path;
+  MountIconSearchData *data = (MountIconSearchData *) (user_data);
+  
+  autorun_path = _g_find_file_insensitive_finish (G_FILE (source_object),
+                                                  res, NULL);
+  if (autorun_path)
+    g_file_load_contents_async (autorun_path, NULL, on_autorun_loaded, data);
+  else
+    clear_icon_search_data (data);
+  
+  g_object_unref (autorun_path);
+}
+
+static void
+_g_find_mount_icon (GHalMount *m)
+{
+  MountIconSearchData *search_data;
+  
+  m->searched_for_icon = TRUE;
+	
+  search_data = g_new0 (MountIconSearchData, 1);
+  search_data->mount = g_object_ref (m);
+  search_data->root = g_mount_get_root (G_MOUNT (m));
+  
+  _g_find_file_insensitive_async (search_data->root,
+                                  "autorun.inf",
+                                  NULL, on_autorun_located,
+                                  search_data);
+}
 
 #define KILOBYTE_FACTOR 1000.0
 #define MEGABYTE_FACTOR (1000.0 * 1000.0)
@@ -217,10 +367,8 @@ do_update_from_hal (GHalMount *m)
 {
   HalDevice *volume;
   HalDevice *drive;
-
   char *name;
   const char *icon_name;
-
   const char *drive_type;
   const char *drive_bus;
   gboolean drive_uses_removable_media;
@@ -231,6 +379,13 @@ do_update_from_hal (GHalMount *m)
   gboolean volume_disc_has_data;
   const char *volume_disc_type;
   gboolean volume_disc_is_blank;
+  gboolean is_audio_player;
+  const char *icon_from_hal;
+  const char *volume_icon_from_hal;
+  const char *name_from_hal;
+  const char *volume_name_from_hal;
+  gboolean is_crypto;
+  gboolean is_crypto_cleartext;
 
   volume = m->device;
   drive = m->drive_device;
@@ -245,12 +400,44 @@ do_update_from_hal (GHalMount *m)
   volume_disc_has_data = hal_device_get_property_bool (volume, "volume.disc.has_data");
   volume_disc_is_blank = hal_device_get_property_bool (volume, "volume.disc.is_blank");
   volume_disc_type = hal_device_get_property_string (volume, "volume.disc.type");
-  
+  is_audio_player = hal_device_has_capability (drive, "portable_audio_player");
+  icon_from_hal = hal_device_get_property_string (drive, "info.desktop.icon");
+  volume_icon_from_hal = hal_device_get_property_string (volume, "info.desktop.icon");
+  name_from_hal = hal_device_get_property_string (drive, "info.desktop.name");
+  volume_name_from_hal = hal_device_get_property_string (volume, "info.desktop.name");
+
+  is_crypto = FALSE;
+  is_crypto_cleartext = FALSE;
+  if (strcmp (hal_device_get_property_string (volume, "volume.fsusage"), "crypto") == 0)
+    {
+      is_crypto = TRUE;
+    }
+  if (strlen (hal_device_get_property_string (volume, "volume.crypto_luks.clear.backing_volume")) > 0)
+    {
+      is_crypto_cleartext = TRUE;
+    }
+
   /*g_warning ("drive_type='%s'", drive_type); */
   /*g_warning ("drive_bus='%s'", drive_bus); */
   /*g_warning ("drive_uses_removable_media=%d", drive_uses_removable_media); */
-  
-  if (strcmp (drive_type, "disk") == 0)
+
+  if (strlen (volume_icon_from_hal) > 0)
+    {
+      icon_name = volume_icon_from_hal;
+    }
+  else if (strlen (icon_from_hal) > 0)
+    {
+      icon_name = icon_from_hal;
+    }
+  else if (is_audio_player)
+    {
+      icon_name = "multimedia-player";
+    }
+  else if (is_crypto || is_crypto_cleartext)
+    {
+      icon_name = "media-encrypted";
+    }
+  else if (strcmp (drive_type, "disk") == 0)
     {
       if (strcmp (drive_bus, "ide") == 0)
         icon_name = "drive-harddisk-ata";
@@ -281,7 +468,15 @@ do_update_from_hal (GHalMount *m)
     icon_name = "drive-harddisk";
 
   
-  if (volume_fs_label != NULL && strlen (volume_fs_label) > 0)
+  if (strlen (volume_name_from_hal) > 0)
+    {
+      name = g_strdup (volume_name_from_hal);
+    }
+  else if (strlen (name_from_hal) > 0)
+    {
+      name = g_strdup (name_from_hal);
+    }
+  else if (volume_fs_label != NULL && strlen (volume_fs_label) > 0)
     name = g_strdup (volume_fs_label);
   else if (volume_is_disc)
     {
@@ -310,6 +505,12 @@ do_update_from_hal (GHalMount *m)
     m->icon = g_object_ref (m->override_icon);
   else
     m->icon = g_themed_icon_new_with_default_fallbacks (icon_name);
+    
+  /* If this is a CD-ROM, begin searching for an icon specified in
+   * autorun.inf.
+  **/
+  if (strcmp (drive_type, "cdrom") == 0 && !m->searched_for_icon)
+    _g_find_mount_icon (m);
 }
 
 
@@ -434,8 +635,7 @@ g_hal_mount_new_for_hal_device    (GVolumeMonitor    *volume_monitor,
 void 
 g_hal_mount_override_name (GHalMount *mount, const char *name)
 {
-  if (mount->override_name != NULL)
-    g_free (mount->override_name);
+  g_free (mount->override_name);
 
   if (name != NULL)
     mount->override_name = g_strdup (name);
@@ -451,7 +651,7 @@ g_hal_mount_override_icon (GHalMount *mount, GIcon *icon)
   if (mount->override_icon != NULL)
     g_object_unref (mount->override_icon);
 
-  if (icon == NULL)
+  if (icon != NULL)
     mount->override_icon = g_object_ref (icon);
   else
     mount->override_icon = NULL;
@@ -469,9 +669,9 @@ g_hal_mount_new (GVolumeMonitor       *volume_monitor,
   HalDevice *drive_device;
   const char *storage_udi;
   GHalMount *mount;
-  
-  /* No volume for mount: Ignore internal things */
-  if (volume == NULL && g_unix_mount_is_system_internal (mount_entry))
+
+  /* If no volume for mount - Ignore internal things */
+  if (volume == NULL && !g_unix_mount_guess_should_display (mount_entry))
     return NULL;
 
   mount = g_object_new (G_TYPE_HAL_MOUNT, NULL);
@@ -521,6 +721,12 @@ g_hal_mount_new (GVolumeMonitor       *volume_monitor,
 
  not_hal:
 
+  if (volume != NULL)
+    {
+      g_object_unref (mount);
+      return NULL;
+    }
+  
   mount->name = g_unix_mount_guess_name (mount_entry);
   mount->icon = g_unix_mount_guess_icon (mount_entry);
 
@@ -915,3 +1121,321 @@ g_hal_mount_register (GIOModule *module)
 {
   g_hal_mount_register_type (G_TYPE_MODULE (module));
 }
+
+#define INSENSITIVE_SEARCH_ITEMS_PER_CALLBACK 100
+ 
+static void
+enumerated_children_callback (GObject *source_object, GAsyncResult *res,
+                              gpointer user_data);
+ 
+static void
+more_files_callback (GObject *source_object, GAsyncResult *res,
+                     gpointer user_data);
+
+static void
+find_file_insensitive_exists_callback (GObject *source_object,
+                                       GAsyncResult *res,
+                                       gpointer user_data);
+
+typedef struct _InsensitiveFileSearchData
+{
+	GFile *root;
+	gchar *original_path;
+	gchar **split_path;
+	gint index;
+	GFileEnumerator *enumerator;
+	GFile *current_file;
+	
+	GCancellable *cancellable;
+	GAsyncReadyCallback callback;
+	gpointer user_data;
+} InsensitiveFileSearchData;
+
+static void
+_g_find_file_insensitive_async (GFile              *parent,
+                                const gchar        *name,
+                                GCancellable       *cancellable,
+                                GAsyncReadyCallback callback,
+                                gpointer            user_data)
+{
+  InsensitiveFileSearchData *data;
+  GFile *direct_file = g_file_get_child (parent, name);
+  
+  data = g_new0 (InsensitiveFileSearchData, 1);
+  data->cancellable = cancellable;
+  data->callback = callback;
+  data->user_data = user_data;
+  data->root = g_object_ref (parent);
+  data->original_path = g_strdup (name);
+  
+  g_file_query_info_async (direct_file, G_FILE_ATTRIBUTE_STANDARD_TYPE,
+			   G_FILE_QUERY_INFO_NONE, G_PRIORITY_DEFAULT,
+			   cancellable,
+			   find_file_insensitive_exists_callback, data);
+  
+  
+}
+
+static void
+clear_find_file_insensitive_state (InsensitiveFileSearchData *data)
+{
+  if (data->root)
+    g_object_unref (data->root);
+  g_free (data->original_path);
+  if (data->split_path)
+    g_strfreev (data->split_path);
+  if (data->enumerator)
+    g_object_unref (data->enumerator);
+  if (data->current_file)
+    g_object_unref (data->current_file);
+  g_free (data);
+}
+
+static void
+find_file_insensitive_exists_callback (GObject *source_object,
+                                       GAsyncResult *res,
+                                       gpointer user_data)
+{
+  GFileInfo *info;
+  InsensitiveFileSearchData *data = (InsensitiveFileSearchData *) (user_data);
+  
+  /* The file exists and can be found with the given path, no need to search. */
+  if ((info = g_file_query_info_finish (G_FILE (source_object), res, NULL)))
+    {
+      GSimpleAsyncResult *simple;
+      
+      simple = g_simple_async_result_new (G_OBJECT (data->root),
+                                          data->callback,
+                                          data->user_data,
+                                          _g_find_file_insensitive_async);
+      
+      g_simple_async_result_set_op_res_gpointer (simple, g_object_ref (source_object), g_object_unref);
+      g_simple_async_result_complete_in_idle (simple);
+      g_object_unref (simple);
+      clear_find_file_insensitive_state (data);
+    }
+  
+  else
+    {
+      data->split_path = g_strsplit (data->original_path, G_DIR_SEPARATOR_S, -1);
+      data->index = 0;
+      data->enumerator = NULL;
+      data->current_file = g_object_ref (data->root);
+      
+      /* Skip any empty components due to multiple slashes */
+      while (data->split_path[data->index] != NULL &&
+             *data->split_path[data->index] == 0)
+        data->index++;
+      
+      g_file_enumerate_children_async (data->current_file,
+                                       G_FILE_ATTRIBUTE_STANDARD_NAME,
+                                       0, G_PRIORITY_DEFAULT,
+                                       data->cancellable,
+                                       enumerated_children_callback, data);
+    }
+  
+  g_object_unref (source_object);
+}
+
+static void
+enumerated_children_callback (GObject *source_object, GAsyncResult *res,
+                              gpointer user_data)
+{
+  GFileEnumerator *enumerator;
+  InsensitiveFileSearchData *data = (InsensitiveFileSearchData *) (user_data);
+  
+  enumerator = g_file_enumerate_children_finish (G_FILE (source_object),
+                                                 res, NULL);
+  
+  if (enumerator == NULL)
+    {
+      GSimpleAsyncResult *simple;
+      GFile *file;
+      
+      simple = g_simple_async_result_new (G_OBJECT (data->root),
+                                          data->callback,
+                                          data->user_data,
+                                          _g_find_file_insensitive_async);
+      
+      file = g_file_get_child (data->root, data->original_path);
+      
+      g_simple_async_result_set_op_res_gpointer (simple, g_object_ref (file), g_object_unref);
+      g_simple_async_result_complete_in_idle (simple);
+      g_object_unref (simple);
+      clear_find_file_insensitive_state (data);
+      return;
+    }
+  
+  data->enumerator = enumerator;
+  g_file_enumerator_next_files_async (enumerator,
+                                      INSENSITIVE_SEARCH_ITEMS_PER_CALLBACK,
+                                      G_PRIORITY_DEFAULT,
+                                      data->cancellable,
+                                      more_files_callback,
+                                      data);
+}
+
+static void
+more_files_callback (GObject *source_object, GAsyncResult *res,
+                     gpointer user_data)
+{
+  InsensitiveFileSearchData *data = (InsensitiveFileSearchData *) (user_data);
+  GList *files, *l;
+  gchar *filename = NULL, *component, *case_folded_name,
+    *name_collation_key;
+  gboolean end_of_files, is_utf8;
+  
+  files = g_file_enumerator_next_files_finish (data->enumerator,
+                                               res, NULL);
+  
+  end_of_files = files == NULL;
+  
+  component = data->split_path[data->index];
+  g_return_if_fail (component != NULL);
+  
+  is_utf8 = (g_utf8_validate (component, -1, NULL));
+  if (is_utf8)
+    {
+      case_folded_name = g_utf8_casefold (component, -1);
+      name_collation_key = g_utf8_collate_key (case_folded_name, -1);
+      g_free (case_folded_name);
+    }
+  
+  else
+    {
+      name_collation_key = g_ascii_strdown (component, -1);
+    }
+  
+  for (l = files; l != NULL; l = l->next)
+    {
+      GFileInfo *info;
+      const gchar *this_name;
+      gchar *key;
+      
+      info = l->data;
+      this_name = g_file_info_get_name (info);
+      
+      if (is_utf8 && g_utf8_validate (this_name, -1, NULL))
+        {
+          gchar *case_folded;
+          case_folded = g_utf8_casefold (this_name, -1);
+          key = g_utf8_collate_key (case_folded, -1);
+          g_free (case_folded);
+        }
+      else
+        {
+          key = g_ascii_strdown (this_name, -1);
+        }
+      
+      if (strcmp (key, name_collation_key) == 0)
+          filename = g_strdup (this_name);
+      g_free (key);
+      
+      if (filename)
+        break;
+    }
+
+  g_list_foreach (files, (GFunc)g_object_unref, NULL);
+  g_list_free (files);
+  g_free (name_collation_key);
+  
+  if (filename)
+    {
+      GFile *next_file;
+      
+      g_file_enumerator_close_async (data->enumerator,
+                                     G_PRIORITY_DEFAULT,
+                                     data->cancellable,
+                                     NULL, NULL);
+      g_object_unref (data->enumerator);
+      data->enumerator = NULL;
+      
+      /* Set the current file and continue searching */
+      next_file = g_file_get_child (data->current_file, filename);
+      g_free (filename);
+      g_object_unref (data->current_file);
+      data->current_file = next_file;
+      
+      data->index++;
+      /* Skip any empty components due to multiple slashes */
+      while (data->split_path[data->index] != NULL &&
+             *data->split_path[data->index] == 0)
+        data->index++;
+      
+      if (data->split_path[data->index] == NULL)
+       {
+          /* Search is complete, file was found */
+          GSimpleAsyncResult *simple;
+          
+          simple = g_simple_async_result_new (G_OBJECT (data->root),
+                                              data->callback,
+                                              data->user_data,
+                                              _g_find_file_insensitive_async);
+          
+          g_simple_async_result_set_op_res_gpointer (simple, g_object_ref (data->current_file), g_object_unref);
+          g_simple_async_result_complete_in_idle (simple);
+          g_object_unref (simple);
+          clear_find_file_insensitive_state (data);
+          return;
+        }
+      
+      /* Continue searching down the tree */
+      g_file_enumerate_children_async (data->current_file,
+                                       G_FILE_ATTRIBUTE_STANDARD_NAME,
+                                       0, G_PRIORITY_DEFAULT,
+                                       data->cancellable,
+                                       enumerated_children_callback,
+                                       data);
+      return;
+    }
+  
+  if (end_of_files)
+    {
+      /* Could not find the given file, abort the search */
+      GSimpleAsyncResult *simple;
+      GFile *file;
+      
+      g_object_unref (data->enumerator);
+      data->enumerator = NULL;
+      
+      simple = g_simple_async_result_new (G_OBJECT (data->root),
+                                          data->callback,
+                                          data->user_data,
+                                          _g_find_file_insensitive_async);
+      
+      file = g_file_get_child (data->root, data->original_path);
+      g_simple_async_result_set_op_res_gpointer (simple, file, g_object_unref);
+      g_simple_async_result_complete_in_idle (simple);
+      g_object_unref (simple);
+      clear_find_file_insensitive_state (data);
+      return;
+    }
+  
+  /* Continue enumerating */
+  g_file_enumerator_next_files_async (data->enumerator,
+                                      INSENSITIVE_SEARCH_ITEMS_PER_CALLBACK,
+                                      G_PRIORITY_DEFAULT,
+                                      data->cancellable,
+                                      more_files_callback,
+                                      data);
+}
+
+static GFile *
+_g_find_file_insensitive_finish (GFile        *parent,
+                                 GAsyncResult *result,
+                                 GError      **error)
+{
+  GSimpleAsyncResult *simple;
+  GFile *file;
+  
+  g_return_val_if_fail (G_IS_SIMPLE_ASYNC_RESULT (result), NULL);
+  
+  simple = G_SIMPLE_ASYNC_RESULT (result);
+  
+  if (g_simple_async_result_propagate_error (simple, error))
+    return NULL;
+  
+  file = G_FILE (g_simple_async_result_get_op_res_gpointer (simple));
+  return g_object_ref (file);
+}
+
